==> rust_like_result-2.4.2.tar.gz: lib/src/result_base.dart <==
import 'dart:async';

abstract class Result<T> {
  T? _val;
  T? get val => _val;

  Error? _err;
  Error? get err => _err;

  bool get isOk => this is Ok;
  bool get isErr => this is Err;

  /// Throws contained `result.err` if `result` is `Err`. Returns `result.val` if `result` is `Ok`.
  T get unwrap {
    if (isErr) {
      throw err!;
    }
    return val!;
  }

  @override
  String toString() => isErr ? 'Err($err)' : 'Ok($val)';

  /// Returns `val` if `result` is `Ok`. Returns `default_value` if `result` is `Err`.
  T unwrapOrDefault(T default_value) {
    if (isErr) {
      return default_value;
    }
    return val!;
  }

  /// Returns `result.val` if `result` is `Ok`. Returns `elseFunc(result.err)` if `result` is `Err`.
  T unwrapOrElse(T Function(Error? e) elseFunc) {
    if (isErr) {
      return elseFunc(err);
    }
    return val!;
  }

  /// If the `Result` is a `Err`, then replaces internal `err` with `newErr` value.
  /// And returns changed result;
  /// If the `Result` is a `Ok`, then return unchanged `Result` as is;
  Result<T> replacePossibleErrorWith(Error newErr) {
    if (isErr) {
      return Err(newErr);
      // _err = newErr;
      // return this;
    }
    return this;
  }

  /// If the Result is a Err, then replaces internal err with result if given function oldErrorReplacer.
  /// Function oldErrorReplacer should take old error and return new error;
  /// And returns changed result;
  /// If the Result is a Ok, then return unchanged Result as is;
  Result<T> replacePossibleError(
      Error Function(Error oldErr) oldErrorReplacer) {
    if (isErr) {
      // _err = oldErrorReplacer(_err!);
      // return this;
      return Err(oldErrorReplacer(_err!));
    }
    return this;
  }

  /// If `result` is `Err` then returns this `Err` as is.
  /// If `result` is `Ok` then returns result of call f(result.val).
  /// If result of call f() is null - returns Err('function returned null').
  /// If result of call f() is `Result` - returns it as is.
  Result pipe(Function(T) f) {
    if (isErr) {
      return this;
    }
    var res = f(unwrap);
    if (res == null) {
      return Err(StateError('function returned null'));
    } else {
      if (res is Error) {
        return Err(res);
      } else {
        if (res is Result) {
          return res;
        } else {
          return Ok(res);
        }
      }
    }
  }
}

class Ok<T> extends Result<T> {
  @override
  final T? _val;
  Ok(this._val) {
    ArgumentError.checkNotNull(val);
  }
}

class Err<T> extends Result<T> {
  @override
  final Error? _err;
  Err(this._err) {
    ArgumentError.checkNotNull(err);
  }
}

/// Calls `Function` f and awaits it.
/// Returns Ok(...) if all right
/// Returns Err(...) otherwise
Future<Result<T>> tryAsResultAsync<T>(FutureOr<T> Function() f) async {
  try {
    var callResult = await f();
    return callResult == null
        ? Err<T>(StateError('async function returned null'))
        : Ok(callResult);
  } catch (e) {
    return Err(StateError('$e'));
  }
}

/// Calls `Function` f
/// Returns Err(...) if call result is null or function call throws error.
/// Otherwise seturns Ok(...).
Result<T> tryAsResultSync<T>(T? Function() f) {
  try {
    var callResult = f();
    return callResult == null
        ? Err<T>(StateError('function returned null'))
        : Ok(callResult);
  } catch (e) {
    return Err(StateError('$e'));
  }
}

/// Calls test(result) and returns result as is if test return true.
/// Otherwise returns Err(ArgumentError(failedTestMsg)).
Result<T> testResult<T>(bool Function(Result<T>) test, Result<T> result,
        String failedTestMsg) =>
    test(result) ? result : Err(ArgumentError(failedTestMsg));

/// If left-side `Result r` is a Err, then returns the same `Result r`.
/// Otherwise return f(r.val).
/// If result of function call `res` is Result - then returns it as is.
/// Otherwise if result of function call `res` == null - returns Err,
/// else returns Ok(res).
extension ResultPiping<T> on Result<T> {
  Result operator |(Function(T) f) => pipe(f);
}

==> rust_like_result-2.5.0.tar.gz: lib/src/result_base.dart <==
import 'dart:async';

abstract class Result<T> {
  T? _val;
  T? get val => _val;

  Error? _err;
  Error? get err => _err;

  bool get isOk => this is Ok;
  bool get isErr => this is Err;

  /// Throws contained `result.err` if `result` is `Err`. Returns `result.val` if `result` is `Ok`.
  T get unwrap {
    if (isErr) {
      throw err!;
    }
    return val!;
  }

  @override
  String toString() => isErr ? 'Err($err)' : 'Ok($val)';

  /// Returns `val` if `result` is `Ok`. Returns `default_value` if `result` is `Err`.
  T unwrapOrDefault(T default_value) {
    if (isErr) {
      return default_value;
    }
    return val!;
  }

  /// Returns `result.val` if `result` is `Ok`. Returns `elseFunc(result.err)` if `result` is `Err`.
  T unwrapOrElse(T Function(Error? e) elseFunc) {
    if (isErr) {
      return elseFunc(err);
    }
    return val!;
  }

  /// If the `Result` is a `Err`, then replaces internal `err` with `newErr` value.
  /// And returns changed result;
  /// If the `Result` is a `Ok`, then return unchanged `Result` as is;
  Result<T> replacePossibleErrorWith(Error newErr) {
    if (isErr) {
      return Err(newErr);
      // _err = newErr;
      // return this;
    }
    return this;
  }

  /// If the Result is a Err, then replaces internal err with result if given function oldErrorReplacer.
  /// Function oldErrorReplacer should take old error and return new error;
  /// And returns changed result;
  /// If the Result is a Ok, then return unchanged Result as is;
  Result<T> replacePossibleError(
      Error Function(Error oldErr) oldErrorReplacer) {
    if (isErr) {
      // _err = oldErrorReplacer(_err!);
      // return this;
      return Err(oldErrorReplacer(_err!));
    }
    return this;
  }

  /// If `result` is `Err` then returns this `Err` as is.
  /// If `result` is `Ok` then returns result of call f(result.val).
  /// If result of call f() is null - returns Err('function returned null').
  /// If result of call f() is `Result` - returns it as is.
  Result<T2> pipe<T2>(Function(T) f) {
    if (isErr) {
      return Err<T2>(err);
    }
    var argument = unwrap;
    var res = f(argument);
    if (res == null) {
      return Err<T2>(
          StateError('Result of f($argument) == null, where `f` is $f'));
    } else {
      if (res is Error) {
        return Err<T2>(res);
      } else {
        if (res is Ok) {
          return Ok<T2>(res.unwrap);
        } else if (res is Err) {
          return Err<T2>(res.err!);
        } else {
          return Ok<T2>(res);
        }
      }
    }
  }

  Future<Result<T2>> pipeAsync<T2>(Future Function(T) f) async {
    if (isErr) {
      return Err<T2>(err);
    }
    var argument = unwrap;
    var res = await f(argument);
    if (res == null) {
      return Err<T2>(
          StateError('Result of f($argument) == null, where `f` is $f'));
    } else {
      if (res is Error) {
        return Err<T2>(res);
      } else {
        if (res is Ok) {
          return Ok(res.unwrap);
        }
        else if (res is Err) {
          return Err<T2>(res.err);
        } else {
          return Ok(res);
        }
      }
    }
  }

  Result<T> test(bool Function(Result<T>) testFunction, String failedTestMsg) =>
      testResult(testFunction, this, failedTestMsg);
}

class Ok<T> extends Result<T> {
  @override
  final T? _val;
  Ok(this._val) {
    ArgumentError.checkNotNull(val);
  }
}

class Err<T> extends Result<T> {
  @override
  final Error? _err;
  Err(this._err) {
    ArgumentError.checkNotNull(err);
  }
}

/// Calls `Function` f and awaits it.
/// Returns Ok(...) if all right
/// Returns Err(...) otherwise
Future<Result<T2>> tryAsResultAsync<T2>(FutureOr<T2?> Function() f) async {
  try {
    var callResult = await f();
    return callResult == null
        ? Err<T2>(StateError('async function returned null'))
        : Ok(callResult);
  } catch (e) {
    return Err(StateError('$e'));
  }
}

/// Calls `Function` f
/// Returns Err(...) if call result is null or function call throws error.
/// Otherwise seturns Ok(...).
Result<T> tryAsResultSync<T>(T? Function() f) {
  try {
    var callResult = f();
    return callResult == null
        ? Err<T>(StateError('function returned null'))
        : Ok(callResult);
  } catch (e) {
    return Err(StateError('$e'));
  }
}

/// Calls test(result) and returns result as is if test return true.
/// Otherwise returns Err(ArgumentError(failedTestMsg)).
Result<T> testResult<T>(bool Function(Result<T>) test, Result<T> result,
        String failedTestMsg) =>
    test(result) ? result : Err(ArgumentError(failedTestMsg));

/// If left-side `Result r` is a Err, then returns the same `Result r`.
/// Otherwise return f(r.val).
/// If result of function call `res` is Result - then returns it as is.
/// Otherwise if result of function call `res` == null - returns Err,
/// else returns Ok(res).
extension ResultPiping<T, T2> on Result<T> {
  Result<T2> operator |(T2 Function(T) f) => pipe(f);
  Future<Result<T2>> operator /(Future<T2> Function(T) f) async =>
      await pipeAsync(f);
}

extension ResultPipingAsync<T> on Future<Result<T>> {
  Future<Result<T2>> pipe<T2>(Function(T) f) async =>
      then((result) => result.pipe(f));

  Future<Result<T2>> pipeAsync<T2>(Future Function(T) f) async {
    return then((resultT) async {
      if (resultT.isErr) {
        return Err<T2>(resultT.err);
      }
      var valueT = resultT.unwrap;
      var res = await f(valueT);
      if (res == null) {
        return Err<T2>(
            StateError('Result of f($valueT) == null, where `f` is $f'));
      } else {
        if (res is Error) {
          return Err<T2>(res);
        } else {
          if (res is Ok) {
            return Ok<T2>(res.val);
          } else if (res is Err) {
            return Err<T2>(res.err);
          } else {
            return Ok(res);
          }
        }
      }
    });
  }
}
